// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#include "account.h"

//robodoc code
//****c* Anvil/Account
//
// NAME
//  Account class
// USAGE
//  #include "account.h"
//  Account a = new Account();
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  11/18/99
// HISTORY
//***
//

Account::Account() { 
}

Account::~Account() {
}

//robodoc code
//****m* Anvil/Account::Name
// NAME
//  Name - the account name, a string.
// SYNOPSIS
//  string Account::Name()
//  void Account::Name(string value)
// PURPOSE
//  Use to get/set the account name.
//  More than one account may be created, and this provides a convenient
//  way to list, search and choose accounts.  This is free and is not
//  linked to any other inputs.
//***
string Account::Name(){
     return name;
}

void Account::Name(string value){
     name = value;
}

//robodoc code
//****m* Anvil/Account::State
// NAME
//  State - the account state, a string.
// SYNOPSIS
//  string Account::State()
//  void Account::State(string value)
// PURPOSE
//  Use to get/set the account state.
//  Accounts can have a state of operation regarding the datas validity.
//  Open accounts are current and working.
//  Closed accounts are inactive but still show in lists for reporting
//    purposes.
//  Inactive accounts are archived.  They are only available for special
//    reporting purposes.  But generally are not visible.
//***
string Account::State(){
     return state;
}

void Account::State(string value){
     //valid input open, closed, or inactive
     state = value;
}

//robodoc code
//****m* Anvil/Account::Type
// NAME
//  Type - the account type, a string.
// SYNOPSIS
//  string Account::Type()
//  void Account::Type(string value)
// PURPOSE
//  Use to get/set the account type.
//  Accounts are typed within Quicken.  This is defined by the first line
//  of the qif file (the ! line item).  Generally speaking we should not 
//  allow changes to the type, but it is a library and we can create
//  new accounts so its here.
//***
string Account::Type(){
     //valid input bank, loan, investment, credit
     return type;
}

void Account::Type(string value){
     if (!value.empty()){
	  if (value == "Invest"){
	       //q = Qif4Invest();
	  }
	  else {
	       q = new Qif4Basic();
	  }
	  type = value;
     }
     else {
	  throw Msg("Type must be a string value.");
     }
}

//robodoc code
//****m* Anvil/Account::Balance
// NAME
//  Balance - the account balance, a float value.
// SYNOPSIS
//  float Account::Balance()
// PURPOSE
//  Use to get the account balance.
//  Account balances are tracked at each transaction (they are a part of 
//  the transaction class).  So whenever a transaction is altered a 
//  calculation occurs from that point to the end of the list to recalc 
//  all the balances.
//***
float Account::Balance(){
     TrBasic *t = (TrBasic*)vtranslist.back();
     return t->Balance();
}

//robodoc code
//****m* Anvil/Account::Size
// NAME
//  Size - the number of transactions in the account, a int value.
// SYNOPSIS
//  int Account::Size()
// PURPOSE
//  Use to get the number of transactions in the account.
//***
int Account::Size(){
     return vtranslist.size();
}

//robodoc code
//****m* Anvil/Account::File
// NAME
//  File - the account filename,  a string value.
// SYNOPSIS
//  string Account::File()
// PURPOSE
//  Use to get the filename that was read or that the account will
//  be written to.
//***
string Account::File(){
     return filename;
}

//robodoc code
//****m* Anvil/Account::Read
// NAME
//  Read - open a qif file and populate the account.
// SYNOPSIS
//  void Account::Read(string file)
// PURPOSE
//  Use to open a qif data file and populate the account
//  transaction list.
// SIDE EFFECTS
//  Sets the filename value returned in Account::File and 
//  also used in Account::Write below.
//***
//read the file and creates a vector of the transactions
void Account::Read(string file){
     if (!file.empty()){
	  if (vtranslist.empty()){
	       filename = file;
	       //get the filetype here by reading the 
	       //first line of the file and testing it.

	       //load the data
	       q->Read(filename, &vtranslist);
	  }
	  else {
	       //should we presume the developer right
	       //and reset the translist here?
	       throw Msg("Error: account list already has data loaded.");
	  }
     }
     else {
	  throw Msg("Error: filename is empty or null value.");
     }
}

//robodoc code
//****m* Anvil/Account::Write
// NAME
//  Write - write the qif file to disk.
// SYNOPSIS
//  void Account::Write()
//  void Account::Write(string file)
// PURPOSE
//  Use to save a qif data file to disk.  If no input is given write
//  will use 'filename' value set internally by the Account::Read
//  method.  Otherwise, a passed in value will be the filename.
//  Normal POSIX path specs apply.
//***
//save any changes to the stated file
//if no file is given use the account filename
void Account::Write(string file){
     if (!file.empty()){
	  q->Write(file, &vtranslist);
     }
     else {
	  q->Write(filename, &vtranslist);
     }
}

//robodoc code
//****m* Anvil/Account::Add
// NAME
//  Add - add a new, blank transaction to the account transaction list.
// SYNOPSIS
//  void Account::Add()
// PURPOSE
//  Use to add a new, blank transaction to the account transaction list.
// TODO
//  jmr...
//  I think we should overload this.  If given a delimited string, then
//  populate the transaction before adding it the the vector. 
//***
void Account::Add(Transaction* value){
     vtranslist.push_back(value);
}

//robodoc code
//****m* Anvil/Account::Update
// NAME
//  Update - updates a transaction in the account
//           with a new transaction object.
// SYNOPSIS
//  void Account::Update(int index, Transaction *trans)
// PURPOSE
//  To update the transaction item in the transaction list maintained 
//  by the account. 
// ERRORS
//  Throws a Msg::Msg exception to the caller. 
// TODO
//  jmr...
//  Implement a method that puts string data into a transaction
//  object.
//***
void Account::Update(int index, Transaction* trans){
     if (!vtranslist.empty()){
	  //create an interator
	  vector<Transaction*>::iterator p = vtranslist.begin();
	  if ((long)index <= vtranslist.size()){
	       for (int k=0; k<index; k++){
		    p++;
	       }
	       //once positioned we are ready to update
	       //first delete the current transaction
	       vtranslist.erase(p);
	       //then insert the one that has new data in the same place
	       p = vtranslist.insert(p, trans);
	  }
	  else {
	       throw Msg("Error: index is beyond bounds of list.");
	  }
     }
     else {
	  throw Msg("Error: Empty transaction list.");
     }
}

//robodoc code
//****m* Anvil/Account::Delete
// NAME
//  Delete - deletes a transaction in the account.
// SYNOPSIS
//  void Account::Delete(int index)
// PURPOSE
//  To delete the transaction item from the account transaction list.
// ERRORS
//  Throws a Msg::Msg exception to the caller. 
// SIDE EFFECTS
//  Calls Account::Rebalance to recalculate the account balances for 
//  each transaction after 'index'.
//***
void Account::Delete(int index){
     if (!vtranslist.empty()){
	  vector<Transaction*>::iterator p = vtranslist.begin();
	  if ((long)index <= vtranslist.size()){
	       for (int k=0; k<index; k++){
		    p++;
	       }
	       vtranslist.erase(p);
	  }
	  else {
	       throw Msg("Error: index is beyond bounds of list.");
	  }
     }
     else {
	  throw Msg("Error: Empty transaction list.");
     }
     //now recalc stack
     Rebalance(index);
}

//robodoc code
//****m* Anvil/Account::Void
// NAME
//  Void - voids a transaction in the account.
// SYNOPSIS
//  void Account::Void(int index)
// PURPOSE
//  To void the transaction item from the account transaction list.
// SIDE EFFECTS
//  Calls Account::Rebalance to recalculate the account balances for 
//  each transaction after 'index'.
//***
void Account::Void(int index){
     //get the transaction
     TrBasic *t = (TrBasic*)Get(index);
     //set the void params
     t->Void();
     //reinsert the updated trans
     Rebalance(index);
     
}

//robodoc code
//****m* Anvil/Account::Get
// NAME
//  Get - retrieves a transaction in the account.
// SYNOPSIS
//  Transaction Account::Get(int index)
// PURPOSE
//  To retrieve a transaction item from the account transaction list.
// RETURN VALUE
//  A copy of a Transaction object.
//  See Tranaction class docs for methods that can be used.
//***
Transaction* Account::Get(int index){
     return vtranslist[index];  
}


//robodoc code
//****m* Anvil/Account::Rebalance
// NAME
//  Rebalance - recalculates the account balance.
// TAGS
//  Private Method - for developers only
// SYNOPSIS
//  void Account::Rebalance(int index)
// PURPOSE
//  To recalculate the balance of each transaction item in the account 
//  transaction list after the point 'index'.
// ERRORS
//  Throws a Msg::Msg exception to the caller. 
//***
//call this private method after changing a transaction
//with the add, delete, update or void methods.
void Account::Rebalance(int index) throw(Msg){
     //Transaction *t;
     TrBasic *t;
     float balance = 0.00;

     //recalc balances from index to end.
     if (!vtranslist.empty()){
	  //create an iterator
	  vector<Transaction*>::iterator p = vtranslist.begin();
	  if ((long)index <= vtranslist.size()){
	       if (index != 0){
		    for (int k=0; k<index-1; k++){
			 p++;
		    }
	       }
	       //set the index -1 values
	       t = (TrBasic*)*p;
	       balance = t->Balance();
	       //once positioned we are ready to update
	       while (p != vtranslist.end()){
		    p++;
		    t = (TrBasic*)*p;
		    t->Balance(balance + t->Amount());
		    balance = t->Balance();
	       }
	       
	  }
	  else {
	       throw Msg("Error: index is beyond bounds of list.");
	  }
     }
     else {
	  throw Msg("Error: Empty transaction list.");
     }
}


