/* 
 * Project: Anvil
 * Personal Finance Application for Linux
 *
 * Joseph M. Robertson, jmr, jmrobert5@mchsi.com
 *
 * Copyright(c) 2001, Joseph M. Robertson.
 * This code protected by the GNU Public License.
 * See the LICENSE file or http://www.gnu.org for details.
 */

/* a C code interface file for 
 * using Anvil class objects
 * within the SLang interpreter.
 *
 * Joe Robertson, jmrobert5@mchsi.com
 *
 */

#include "sl_int.h"


/* robodoc code */
/****c* Anvil/sl_int
 *
 * NAME
 *  S-Lang script interface to Anvil
 * USAGE
 *  #include "sl_int.h"
 *  if (-1 == SLadd_intrin_fun_table (Acc_Intrinsics, NULL)){
 *	exit (EXIT_FAILURE);
 *  }	
 * AUTHOR
 *  Joseph M. Robertson, jmr, jmrobert5@home.com
 * COPYRIGHT
 * (c) 2001 by Joseph M. Robertson
 * CREATION DATE
 *  3/18/01
 * HISTORY
 *  6/21/01  added acc_get/set_type functions.
 * SEE ALSO
 *  The file sl_main.cpp for an example of how to embed S-Lang
 *  into another application and include Anvil functions.
 ******
 */

vector<Account> v_acc;
Account *acc;
string l_string;

/* robodoc code */
/****f* Anvil/sl_int
 *
 * NAME
 *  acc_new()
 * SYNOPSIS
 *  variable response;
 *  response = acc_new();
 * PURPOSE
 *  Use to create a new account space in slang.
 *  sl_int functions 'work' on the currently set account.
 * RETURN VALUE
 *  The number of accounts in the current instance.
 ******
 */
int intrin_acc_new(){
     acc = new Account();
     v_acc.push_back(*acc);
     return v_acc.size();
}

/* robodoc code */
/****f* Anvil/sl_int
 *
 * NAME
 *  acc_set
 * SYNOPSIS
 *  variable response;
 *  response = acc_set(1);
 * PURPOSE
 *  Use to set the working account space in slang.
 *  sl_int functions 'work' on the currently set account.
 * RETURN VALUE
 *  The number of the account in the current instance.
 *  Or zero if no accounts exist.
 ******
 */
int intrin_acc_set(int *index){
     int l_index = *index;
     
     if (!v_acc.empty()){
	  *acc = v_acc[l_index];
     }
     else { 
	  acc = NULL;
	  l_index = 0;
     }
     return l_index;
}

/* robodoc code */
/****f* Anvil/sl_int
 *
 * NAME
 *  acc_close
 * SYNOPSIS
 *  variable response;
 *  response = acc_close(1);
 * PURPOSE
 *  Use to close and remove the working account space in slang.
 *  sl_int functions 'work' on the currently set account.
 * RETURN VALUE
 *  The index of the account closed OR
 *  -1 if the account index is not found OR
 *  -2 if the account list was empty to start.
 ******
 */
int intrin_acc_close(int *index){
     int l_index = *index;
     
     if (!v_acc.empty()){
	  vector<Account>::iterator p = v_acc.begin();
	  if(l_index <= v_acc.size()){
	       for (int k=0; k<l_index; k++){
		    p++;
	       }
	       v_acc.erase(p);
	  }
	  else {
	       return -1;
	  }
     }
     else { 
	  acc = NULL;
	  l_index = -2;
     }
     return l_index;

     
}

/*account functions exposed*/
int intrin_acc_read(char *file){
     int response = 0;

     if (acc == NULL) {
	  response = intrin_acc_new();
     }
     string s_file = file;
     try {
	  acc->Read(s_file);
     }
     catch(Msg &msg){
	  cout << msg.Get().c_str() << endl;
     }
     return response;
	
}

void intrin_acc_write(char *file){
     string s_file = file;
	acc->Write(s_file);
}

int intrin_acc_size(){
	return acc->Size();
}

char *intrin_acc_get_type(){
     return (char *)acc->Type().c_str();
}

void intrin_acc_set_type(char *type){
     string s_type = type;
     acc->Type(s_type);
}


char *intrin_acc_get_state(){
     return (char *)acc->State().c_str();
}

void intrin_acc_set_state(char *state){
     string s_state = state;
     acc->State(s_state);
}

void intrin_acc_add(){
     Transaction *t = new Transaction();
     acc->Add(t);
}

void intrin_acc_delete(int *index){
     int l_index = *index;
     try{
	  acc->Delete(l_index);
     }
     catch(Msg &msg){
	  cout << msg.Get().c_str() << endl;
     }
}

void intrin_acc_void(int *index){
     int l_index = *index;
     acc->Void(l_index);
}

char *intrin_acc_get_transaction(int *index){
     int l_index = *index;

     /* change to a pipe delim string, in transacation */
     if (strcmp(intrin_acc_get_type(), "Invest") == 0) {
	  /* use invest transaction */
	  TrBasic *t = (TrBasic*)acc->Get(l_index);
	  l_string = t->Print_Rec();
     }
     else {
	  TrBasic *t = (TrBasic*)acc->Get(l_index);
	  l_string = t->Print_Rec();
     }

     return (char *)l_string.c_str();
}

void intrin_acc_set_transaction(char *trans){
     /* insert a pipe delimited string here */
}


char *intrin_acc_get_name(){
     return (char *)acc->Name().c_str();
}

void intrin_acc_set_name(char *name){
     string s_name = name;
     acc->Name(s_name);
}

char *intrin_acc_field(int *index, int *field){
     int l_index = *index;
     int l_field = *field;


     if (strcmp(intrin_acc_get_type(), "Invest") == 0) {
	  /* use invest transaction */
	  TrBasic *t = (TrBasic*)acc->Get(l_index);
	  l_string = t->Field(l_field);
     }
     else {
	  TrBasic *t = (TrBasic*)acc->Get(l_index);
	  l_string = t->Field(l_field);
     }

     return (char *)l_string.c_str();
}

