// 
// Project: Anvil
// Personal Finance Application for Linux
//
// Joseph M. Robertson, jmr, jmrobert5@mchsi.com
//
// Copyright(c) 2001, Joseph M. Robertson.
// This code protected by the GNU Public License.
// See the LICENSE file or http://www.gnu.org for details.
//

#include "trbasic.h"

//robodoc code
//****c* Anvil/TrBasic
//
// NAME
//  Transaction Basic class
// USAGE
//  #include "trbasic.h"
//  TrBasic *t = new TrBasic();
// AUTHOR
//  Joseph M. Robertson, jmr, jmrobert5@mchsi.com
// COPYRIGHT
// (c) 2001 by Joseph M. Robertson
// CREATION DATE
//  6/17/01
// HISTORY
//***
//


TrBasic::TrBasic(){
     amount = 0.00;
     number = 0;
     address = "";
     category = "";
     balance = 0.00;
     vector<Split> split;
}

TrBasic::~TrBasic(){
     //cout << "freeing trans." << endl;
}

//robodoc code
//****m* Anvil/TrBasic::Amount
// NAME
//  Amount - the dollar amount of the transaction.
// SYNOPSIS
//  float TrBasic::Amount()
//  void TrBasic::Amount(float value)
// PURPOSE
//  Use to get/set the dollar amount of the transaction.
//***
float TrBasic::Amount(){
     return amount;
}

void TrBasic::Amount(float value){
     amount = value;
}



//robodoc code
//****m* Anvil/TrBasic::Void
// NAME
//  Void
// SYNOPSIS
//  void TrBasic::Void()
// PURPOSE
//  Convenience method to void a transaction.
//  Sets status to 'x', ammount to 0.00 and payee to **VOID**
//***
void TrBasic::Void(){
     Status("x");
     Payee("** VOID **");
     amount = 0.00;
}

//robodoc code
//****m* Anvil/TrBasic::Number
// NAME
//  Number
// SYNOPSIS
//  long TrBasic::Number()
//  void TrBasic::Number(long value)
// PURPOSE
//  Use to get/set the check number for the transaction.
//***
long TrBasic::Number(){
     return number;
}

void TrBasic::Number(long value){
     number= value;
}


//robodoc code
//****m* Anvil/TrBasic::Address
// NAME
//  Address
// SYNOPSIS
//  string TrBasic::Address()
//  void TrBasic::Address(string value)
// PURPOSE
//  Use to get/set the address member of the transaction.
//***
string TrBasic::Address(){
     return address;
}

//address can be multi-lined
void TrBasic::Address(string value){
     address += value;
}

//robodoc code
//****m* Anvil/TrBasic::Category
// NAME
//  Category
// SYNOPSIS
//  string TrBasic::Category()
//  void TrBasic::Category(string value)
// PURPOSE
//  Use to get/set the category name member of the transaction.
//***
string TrBasic::Category(){
     return category;
}

void TrBasic::Category(string value){
     category = value;
}

//robodoc code
//****m* Anvil/TrBasic::Balance
// NAME
//  Balance
// SYNOPSIS
//  float TrBasic::Balance()
//  void TrBasic::Balance(float value)
// PURPOSE
//  Use to get/set the balance. Transactions as a linked list or vector
//  can be used to track the account balance.
//***
float TrBasic::Balance(){
     return balance;
}

void TrBasic::Balance(float value){
     balance = value;
}

//robodoc code
//****m* Anvil/TrBasic::VSplit
// NAME
//  VSplit
// SYNOPSIS
//  vector<Split> TrBasic::VSplit()
//  void TrBasic::VSplit(vector<Split> vsplit)
// PURPOSE
//  Use to get/set the vector of splits that are optional to a transaction.
//  Splits are a way to subdivide the details of a transaction.
//***
vector<Split> TrBasic::VSplit(){
     return split;
}

void TrBasic::VSplit(vector<Split> vsplit){
     split = vsplit;
     ReBalance();
}

//robodoc code
//****m* Anvil/TrBasic::Print()
// NAME
//  Print
// SYNOPSIS
//  string TrBasic::Print()
// PURPOSE
//  Use to get a formatted string of the transaction.
//  This is a verbose version.
//***
string TrBasic::Print(){
     string temp;
     string s_id, s_number, s_amount;
     char num[12];
     
     //translate numeric values to string objects
     sprintf(num, "%ld", ID());
     s_id = num;
     sprintf(num, "%ld", number);
     s_number = num;
     sprintf(num, "%8.2f", amount);
     s_amount = num;

     //populate return string
     temp.append("----------\n");
     temp.append("ID: " + s_id + "\n");
     temp.append("Date: " + Date() + "\n");
     temp.append("Number: " + s_number + "\n");
     temp.append("Status: " + Status() + "\n");
     temp.append("Amount: " + s_amount + "\n");
     
     temp.append("Payee: " + Payee() + "\n");
     temp.append("Memo: " + Memo() + "\n");
     temp.append("Address: " + address + "\n");
     temp.append("Category: " + category + "\n");

     if (split.size() > 0) {
	  temp.append("Split Values:\n");
	  //create an iterator
	  vector<Split>::iterator p = split.begin();
	  for (int k=0; k<split.size(); k++){
	       //get values from item
	       sprintf(num, "%8.2f", p->Amount());
	       s_amount = num;
	       temp.append("\tAmount: " + s_amount + "\n");
	       temp.append("\tMemo: " + p->Memo() + "\n");
	       temp.append("\tCategory: " + p->Category() + "\n");
	       //increment pointer
	       p++;
	  }
     }
     temp.append("----------\n");

     return temp;
}

//robodoc code
//****m* Anvil/TrBasic::Print_Rec()
// NAME
//  Print_Rec
// SYNOPSIS
//  string TrBasic::Print_Rec()
// PURPOSE
//  Use to print out a string format of the transaction data.
//  This is a short format, pipe delimited string.
//***
string TrBasic::Print_Rec(){
     string temp = "";
     string s_id, s_number, s_amount;
     char num[12];
     string delim = "|";

     //translate numeric values to string objects
     sprintf(num, "%ld", ID());
     s_id = num;
     sprintf(num, "%ld", number);
     s_number = num;
     sprintf(num, "%8.2f", amount);
     s_amount = num;

     //populate return string
     temp.append(s_id + delim);
     temp.append(Date() + delim);
     temp.append(s_number + delim);
     temp.append(Status() + delim);
     temp.append(s_amount + delim);
     temp.append(Payee() + delim);
     temp.append(Memo() + delim);
     temp.append(address + delim);
     temp.append(category + delim);

     if (split.size() > 0) {
	  temp.append("%" + delim);
	  //create an iterator
	  vector<Split>::iterator p = split.begin();
	  for (int k=0; k<split.size(); k++){
	       //get values from item
	       sprintf(num, "%8.2f", p->Amount());
	       s_amount = num;
	       temp.append(s_amount + delim);
	       temp.append(p->Memo() + delim);
	       temp.append(p->Category() + delim);
	       //increment pointer
	       p++;
	  }
     }
     return temp;
}

//robodoc code
//****m* Anvil/TrBasic::Field()
// NAME
//  Field
// SYNOPSIS
//  string TrBasic::Field(int index)
// PURPOSE
//  Use to get a piece of member data from a transaction.
//  0 = id         6 = memo
//  1 = date       7 = address (could contain linefeeds)
//  2 = number     8 = category
//  3 = status     9 = split  (pipe delimited string)
//  4 = amount         amount|memo|category...
//  5 = payee      10 = balance
//***
//return a string field value by index
string TrBasic::Field(int index){
     string temp = "";
     string s_id, s_number, s_amount, s_balance;
     char num[12];
     string delim = "|";

     //translate numeric values to string objects
     sprintf(num, "%ld", ID());
     s_id = num;
     sprintf(num, "%ld", number);
     s_number = num;
     sprintf(num, "%8.2f", amount);
     s_amount = num;
     sprintf(num, "%8.2f", balance);
     s_balance = num;

     //populate return string
     switch (index) {
     case 0:
	  temp = s_id;
	  break;
     case 1:
	  temp = Date();
	  break;
     case 2:
	  temp = s_number;
	  break;
     case 3:
	  temp = Status();
	  break;
     case 4:
	  temp = s_amount;
	  break;
     case 5:
	  temp = Payee();
	  break;
     case 6:
	  temp = Memo();
	  break;
     case 7:
	  temp = address;
	  break;
     case 8:
	  temp = category;
	  break;
     case 9:
	  if (split.size() > 0) {
	       //temp.append("%" + delim);
	       //create an iterator
	       vector<Split>::iterator p = split.begin();
	       for (int k=0; k<split.size(); k++){
		    //get values from item
		    sprintf(num, "%8.2f", p->Amount());
		    s_amount = num;
		    temp.append(s_amount + delim);
		    temp.append(p->Memo() + delim);
		    temp.append(p->Category() + delim);
		    //increment pointer
		    p++;
	       }
	  }
	  break;
     case 10:
	  temp = s_balance;
	  break;
     default:
	  break;
     }
     return temp;
}

//robodoc code
//****m* Anvil/TrBasic::ReBalance
// NAME
//  ReBalance
// SYNOPSIS
//  void TrBasic::ReBalance()
// PURPOSE
//  Used to recalculate the transaction balance when used in a list or vector.
// NOTES
//  Private method.
//***
//call this private method after changing a transaction split
void TrBasic::ReBalance(){
     float balance = 0.00;

     //recalc balances from index to end.
     if (!split.empty()){
	  //create an iterator
	  vector<Split>::iterator p = split.begin();

	  //once positioned we are ready to update
	  while (p != split.end()){
	       balance += p->Amount();
	       p++;
	  }
     }
     else {
	  throw Msg("Error: Empty transaction list.");
     }

}

